﻿using System.Windows.Forms.VisualStyles;
using Infragistics.Windows.Controls;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace pageup_down
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void HandlePreviewKeyDown(object sender, KeyEventArgs e)
        {
            XamTabControl tab = (sender as XamTabControl);
            var sIndex = tab.SelectedIndex;

            if (e.Key == Key.PageDown)
            {
                if (sIndex >= 0 && sIndex < tab.Items.Count - 1)
                {
                    tab.SelectedIndex = sIndex + 1;
                }      
            }
            else if (e.Key == Key.PageUp)
            {
                if (sIndex >= 1 && sIndex <= tab.Items.Count - 1)
                {
                    tab.SelectedIndex = sIndex - 1;
                }
            }

            if (e.Key == Key.Tab && 
                (Keyboard.Modifiers & ModifierKeys.Control) == ModifierKeys.Control)
            {
                e.Handled = true;
            }
        }

        /// <summary>
        /// Not PREVIEW key down.
        /// </summary>
        /// <param name="aSender"></param>
        /// <param name="anEvt"></param>
        private void HandleKeyDown( object aSender, KeyEventArgs anEvt )
        {
            XamTabControl tabCtrl;
            if (aSender is XamTabControl)
                tabCtrl = aSender as XamTabControl;
            else if (aSender is TabItemEx)
                tabCtrl = findParent<XamTabControl>( aSender as TabItemEx );
            else
                tabCtrl = null;
            if (tabCtrl == null) {}
            else
            {
                bool isCtrl = (Keyboard.Modifiers & ModifierKeys.Control) == ModifierKeys.Control;
                if (isCtrl)
                {
                    int? selectedIndex;
                    if (anEvt.Key == Key.PageDown)
                        selectedIndex = tabCtrl.SelectedIndex + 1;
                    else if (anEvt.Key == Key.PageUp)
                        selectedIndex = tabCtrl.SelectedIndex - 1;
                    else
                        selectedIndex = null;
                    if (selectedIndex == null) {}
                    else
                    {
                        if (selectedIndex < 0)
                            selectedIndex = tabCtrl.Items.Count - 1;
                        else if (selectedIndex >= tabCtrl.Items.Count)
                            selectedIndex = 0;
                        tabCtrl.SelectedIndex = selectedIndex.Value;
                        anEvt.Handled = true;
                    }
                }
            }
            if (anEvt.Handled) {}
            else
                base.OnKeyDown( anEvt);
        }

        private T findParent<T>( DependencyObject aChild ) where T : DependencyObject
        {
            DependencyObject parent = VisualTreeHelper.GetParent( aChild );
            if (parent == null || parent is T)
                return parent as T;
            else
                return findParent<T>( parent );
        }
    
        private void TabContentGrid_OnKeyDown( object aSender, KeyEventArgs anEvt )
        {
            if (anEvt.Key == Key.Tab
                && (Keyboard.Modifiers & ModifierKeys.Control) == ModifierKeys.Control)
            {
                var focusedElt = Keyboard.FocusedElement;
                if (focusedElt is TextBox)
                {
                    FocusNavigationDirection focusNavDir;
                    if ((Keyboard.Modifiers & ModifierKeys.Shift) == ModifierKeys.Shift)
                        focusNavDir = FocusNavigationDirection.Previous;
                    else
                        focusNavDir = FocusNavigationDirection.Next;
                    TraversalRequest traversalRequest = new TraversalRequest( focusNavDir);
                    bool focusMoved = ((TextBox)focusedElt).MoveFocus(traversalRequest);
                    anEvt.Handled = focusMoved;
                }
            }
        }

        private void MainWindow_OnLostFocus(object aSender, RoutedEventArgs anEvt)
        {
            var uiElt = anEvt.OriginalSource as UIElement;
            if (uiElt == null) { }
            else
            {
                var adornerLayer = AdornerLayer.GetAdornerLayer(uiElt);
                if (adornerLayer == null) { }
                else
                {
                    var adorners = adornerLayer.GetAdorners(uiElt);
                    if (adorners == null) { }
                    else
                        foreach (var adorner in adorners.Where(a => a is FocusAdorner))
                            adornerLayer.Remove(adorner);
                }
            }

        }

        private void MainWindow_OnGotFocus(object aSender, RoutedEventArgs anEvt)
        {
            var uiElt = anEvt.OriginalSource as UIElement;
            if (uiElt == null) { }
            else
            {
                var adornerLayer = AdornerLayer.GetAdornerLayer(uiElt);
                if (adornerLayer == null) { }
                else
                    adornerLayer.Add(new FocusAdorner(uiElt));
            }
        }
    }
}
